/**
 * This script is used to create a new tag for the project.
 * It will get the last tag, bump the version and create a new tag with the commit log since the last tag.
 * Optionally, it will push the new tag to GitHub.
 *
 * Usage: `npm run tag <'patch' | 'minor' | 'major' | 'rc'>
 *
 * Example: if the last tag is v1.0.0, running `npm run tag patch` will create a new tag v1.0.1
 * Example: if the last tag is v1.3.8rc2, running `npm run tag rc` will create a new tag v1.3.8rc3
 * Example: if the last tag is v1.3.8rc2, running `npm run tag minor` will create a new tag v1.4.0
 * etc.
 */
import { execSync } from 'child_process';
import readline from 'readline';
import chalk from 'chalk';
import { STVersion } from './STVersion.js';
async function main() {
    const versionType = process.argv[2];
    if (!versionType ||
        !['patch', 'minor', 'major', 'rc'].includes(versionType)) {
        console.log(chalk.red('Please provide a valid version type (patch, minor, major or rc)'));
        process.exit(1);
    }
    // Get the last git tag
    const lastTag = execSync('git describe --tags --abbrev=0')
        .toString()
        .trim();
    // Get the commit log since the last tag
    const commitLog = execSync(`git shortlog ${lastTag}..HEAD`)
        .toString()
        .trim();
    const v = new STVersion(lastTag);
    const newVersion = v.bump(versionType);
    // Display current tag and new tag to be created
    console.log(chalk.cyanBright('Current tag:'), chalk.yellowBright.bold(lastTag));
    console.log(chalk.cyanBright('New tag:'), chalk.yellowBright.bold(newVersion));
    console.log(chalk.cyanBright('\nCommit log since last tag:'));
    console.log(chalk.yellow.italic(commitLog) + '\n');
    console.log('----------------------------------');
    try {
        // Ask user for confirmation
        let answer = await prompt('Do you want to create the new tag? (Y/n): ');
        ``;
        if (answer.toLowerCase() !== 'n') {
            // Create new tag with description
            const description = `Version ${newVersion}:\n${commitLog}`;
            execSync(`git tag -a ${newVersion} -m "${description}"`);
            console.log(chalk.greenBright(`Tag ${newVersion} created.\n`));
        }
        else {
            console.log('Tag creation canceled.\n');
        }
        answer = await prompt(`Do you want to push the tag ${newVersion} to GitHub? (Y/n): `);
        if (answer.toLowerCase() !== 'n') {
            execSync(`git push origin ${newVersion}`);
            console.log(chalk.greenBright(`Tag ${newVersion} pushed to GitHub.`));
        }
        else {
            console.log('Tag push canceled.');
        }
    }
    catch (error) {
        console.error('Error occurred:', error);
        process.exit(1);
    }
}
// Function to prompt the user and get input
async function prompt(question) {
    const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout,
    });
    return new Promise((resolve) => rl.question(question, (ans) => {
        rl.close();
        resolve(ans);
    }));
}
main();
